@@ -3,7 +3,7 @@
  *  This file is part of kalarmprivate library, which provides access to KAlarm
  *  calendar data.
  *  Program:  kalarm
- *  SPDX-FileCopyrightText: 2001-2025 David Jarvie <djarvie@kde.org>
+ *  SPDX-FileCopyrightText: 2001-2026 David Jarvie <djarvie@kde.org>
  *
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
@@ -196,27 +196,28 @@ public:
     /** Flags for use in D-Bus calls, etc. Flags may be combined by OR'ing them together. */
     enum Flag
     {
-        BEEP            = 0x02,    //!< sound an audible beep when the alarm is displayed
-        REPEAT_AT_LOGIN = 0x04,    //!< repeat the alarm at every login
-        ANY_TIME        = 0x08,    //!< only a date is specified for the alarm, not a time
-        CONFIRM_ACK     = 0x10,    //!< closing the alarm message window requires a confirmation prompt
-        EMAIL_BCC       = 0x20,    //!< blind copy the email to the user
-        DEFAULT_FONT    = 0x40,    //!< use the default alarm message font. Overrides any specified font.
-        REPEAT_SOUND    = 0x80,    //!< repeat the sound file while the alarm is displayed
-        DISABLED        = 0x100,   //!< the alarm is currently disabled
-        AUTO_CLOSE      = 0x200,   //!< auto-close the alarm window after the late-cancel period
-        SCRIPT          = 0x400,   //!< the command is a script, not a shell command line
-        EXEC_IN_XTERM   = 0x800,   //!< execute the command in a terminal window
-        SPEAK           = 0x1000,  //!< speak the message when the alarm is displayed
-        COPY_KORGANIZER = 0x2000,  //!< KOrganizer should hold a copy of the event
-        EXCL_HOLIDAYS   = 0x4000,  //!< don't trigger the alarm on holidays. Only valid
-                                   //!< if a holiday region has been set by setHolidays().
-        WORK_TIME_ONLY  = 0x8000,  //!< trigger the alarm only during working hours
-        DISPLAY_COMMAND = 0x10000, //!< display command output in the alarm window
-        REMINDER_ONCE   = 0x20000, //!< only trigger the reminder on the first recurrence
-        DONT_SHOW_ERROR = 0x40000, //!< do not notify command alarm errors to user
-        NOTIFY          = 0x80000, //!< use the standard notification system instead of alarm message window
-        WAKE_SUSPEND    = 0x100000 //!< use kernel timer (not RTC) to wake-from-suspend when alarm due
+        Beep            = 0x02,     //!< sound an audible beep when the alarm is displayed
+        RepeatAtLogin   = 0x04,     //!< repeat the alarm at every login
+        AnyTime         = 0x08,     //!< only a date is specified for the alarm, not a time
+        ConfirmAck      = 0x10,     //!< closing the alarm message window requires a confirmation prompt
+        EmailBcc        = 0x20,     //!< blind copy the email to the user
+        DefaultFont     = 0x40,     //!< use the default alarm message font. Overrides any specified font.
+        RepeatSound     = 0x80,     //!< repeat the sound file while the alarm is displayed
+        Disabled        = 0x100,    //!< the alarm is currently disabled
+        AutoClose       = 0x200,    //!< auto-close the alarm window after the late-cancel period
+        Script          = 0x400,    //!< the command is a script, not a shell command line
+        ExecInXterm     = 0x800,    //!< execute the command in a terminal window
+        Speak           = 0x1000,   //!< speak the message when the alarm is displayed
+        CopyKOrganizer  = 0x2000,   //!< KOrganizer should hold a copy of the event
+        ExcludeHolidays = 0x4000,   //!< don't trigger the alarm on holidays. Only valid
+                                    //!< if a holiday region has been set by setHolidays().
+        WorkTimeOnly    = 0x8000,   //!< trigger the alarm only during working hours
+        DisplayCommand  = 0x10000,  //!< display command output in the alarm window
+        ReminderOnce    = 0x20000,  //!< only trigger the reminder on the first recurrence
+        DontShowError   = 0x40000,  //!< do not notify command alarm errors to user
+        Notify          = 0x80000,  //!< use the standard notification system instead of alarm message window
+        WakeSuspend     = 0x100000, //!< use kernel timer (not RTC) to wake-from-suspend when alarm due
+        NoInhibit       = 0x200000  //!< execute alarm regardless of notification inhibition
 
                           // IMPORTANT: if any values are added to this list, ensure that the
                           //            additional enum values in KAEventPrivate are also adjusted.
@@ -233,6 +234,7 @@ public:
         Email          = 0x04,   //!< the alarm sends an email
         Audio          = 0x08,   //!< the alarm plays an audio file (without any display)
         DisplayCommand = Display | Command,  //!< the alarm displays command output
+        Notification   = Display | Audio,    //!< mask for a notification type alarm
         All            = Display | Command | Email | Audio   //!< all types mask
     };
 
@@ -372,6 +374,7 @@ public:
      *  of NextType.
      *  Reminders are never returned if the recurrence to which they relate is excluded by working
      *  hours or holiday restrictions, regardless of whether or not NextWorkHoliday is specified.
+     *  Reminders are never returned for skipped occurrences, if NextSkip is specified.
      */
     enum NextType
     {
@@ -379,7 +382,8 @@ public:
         NextRepeat      = 0x01,   //!< check for sub-repetitions
         NextReminder    = 0x02,   //!< check for reminders
         NextWorkHoliday = 0x04,   //!< take account of any working hours or holiday restrictions
-        NextDeferral    = 0x08    //!< return the event deferral time, or reminder deferral time if NextReminder set
+        NextDeferral    = 0x08,   //!< return the event deferral time, or reminder deferral time if NextReminder set
+        NextSkip        = 0x10    //!< take account of skipping
     };
     Q_DECLARE_FLAGS(NextTypes, NextType)
 
@@ -387,28 +391,32 @@ public:
     enum class Trigger
     {
 
-        /** Next trigger, including reminders. No account is taken of any
-         *  working hours or holiday restrictions when evaluating this. */
+        /** Next trigger, including reminders. No account is taken of any working
+         *  hours or holiday restrictions, or skipping, when evaluating this. */
         All,
 
         /** Next trigger of the main alarm, i.e. excluding reminders. No
-         *  account is taken of any working hours or holiday restrictions when
-         *  evaluating this. */
+         *  account is taken of any working hours or holiday restrictions, or
+         *  skipping, when evaluating this. */
         Main,
 
         /** Next trigger of the main alarm, i.e. excluding reminders, taking
-         *  account of any working hours or holiday restrictions. If the event
-         *  has no working hours or holiday restrictions, this is equivalent to
-         *  Main. */
+         *  account of any working hours or holiday restrictions. No account is
+         *  taken of skipping. If the event has no working hours or holiday
+         *  restrictions, this is equivalent to Main. */
         Work,
 
         /** Next trigger, including reminders, taking account of any working
-         *  hours or holiday restrictions. If the event has no working hours or
-         *  holiday restrictions, this is equivalent to All. */
+         *  hours or holiday restrictions. No account is taken of skipping. If
+         *  the event has no working hours or holiday restrictions, this is
+         *  equivalent to All. */
         AllWork,
 
-        /** Next trigger time for display purposes (i.e. excluding reminders). */
-        Display
+        /** Next time the alarm will actually trigger, i.e. the next recurrence
+         *  or sub-repetition, taking account of working hours and holiday
+         *  restrictions and skipping, but excluding reminders except if it is
+         *  a reminder which occurs AFTER the last recurrence and sub-repetition. */
+        Actual
     };
 
     /** Command execution error type for last time the alarm was triggered. */
@@ -453,7 +461,7 @@ public:
      *  @param bg    background color (for display alarms, ignored otherwise).
      *  @param fg    foreground color (for display alarms, ignored otherwise).
      *  @param font  font (for display alarms, ignored otherwise). Ignored if
-     *               #DEFAULT_FONT flag is specified.
+     *               #DefaultFont flag is specified.
      *  @param action         alarm action type.
      *  @param lateCancel     late-cancellation period (minutes), else 0.
      *  @param flags          OR of #Flag enum values.
@@ -951,10 +959,84 @@ public:
     /** Return the default date-only setting used in the deferral dialog. */
     bool deferDefaultDateOnly() const;
 
+    /** Return the maximum skip count.
+     *  A limit is set in order to prevent excessive processing.
+     */
+    static int maxSkipCount();
+
+    /** Set a number of times for the event to be skipped.
+     *  This is the count of recurrences and sub-repetitions to skip.
+     *  Do not include reminders in the count; these will automatically be
+     *  skipped if their related recurrence or sub-repetition is skipped.
+     *  Skipping does not affect any outstanding deferral of the alarm.
+     *
+     *  Note that the date/time that triggering of the event will resume
+     *  is calculated when this function is called. If the alarm is
+     *  subject to working hours or holiday restrictions and a change is
+     *  later made to working hours or holiday settings, the date/time
+     *  that triggering of the event will resume will not be recalculated
+     *  to comply with the new settings.
+     *
+     *  @param count  number of times to skip the event trigger. If zero,
+     *                skipping will be cancelled.
+     *  @return true if the event is now skipping, false if not.
+     *
+     *  @see cancelSkip(), skipping(), skipCount(), skipDateTime()
+     */
+    bool skip(int count);
+
+    /** Set a time up until when the event should be skipped.
+     *  The event will resume normal activation at the first recurrence
+     *  or sub-repetition on or after this time.
+     *  Any reminder for the first activation after skipping will be
+     *  output even if this is before the time specified.
+     *  Skipping does not affect any outstanding deferral of the alarm.
+     *
+     *  Note that the date/time that triggering of the event will resume
+     *  is set when this function is called. If the alarm is
+     *  subject to working hours or holiday restrictions and a change is
+     *  later made to working hours or holiday settings, the date/time
+     *  that triggering of the event will resume will not be recalculated
+     *  to comply with the new settings.
+     *
+     *  @param dt  date/time on or after which skipping will cease.
+     *             If invalid, skipping will be cancelled.
+     *  @return true if the event is now skipping, false if not.
+     *
+     *  @see cancelSkip(), skipping(), skipCount(), skipDateTime()
+     */
+    bool skip(const KADateTime& dt);
+
+    /** Cancel any skipping which is currently set.
+     *  @see skip()
+     */
+    void cancelSkip();
+
+    /** Return whether the event is currently being skipped.
+     *  @see skip(), skipDateTime(), skipCount()
+     */
+    bool skipping() const;
+
+    /** Return whether the event is currently being skipped, and if so how
+     *  many occurrences remain to be skipped.
+     *  @return  number of event triggers remaining to be skipped. Reminders
+     *           are not included in this count.
+     *  @see skip(), skipDateTime(), skipping()
+     */
+    int skipCount() const;
+
+    /** Return the time at which the event should resume normal triggering.
+     *  The next trigger for the alarm will occur at or after this time.
+     *  @return  time when triggers will resume, or invalid if not currently
+     *           being skipped.
+     *  @see skip(), skipping(), skipCount()
+     */
+    DateTime skipDateTime() const;
+
     /** Return the start time for the event. If the event recurs, this is the
      *  time of the first recurrence. If the event is date-only, this returns a
      *  date-only value.
-     *  @note No account is taken of any working hours or holiday restrictions.
+     *  @note No account is taken of any working hours or holiday restrictions
      *        when determining the start date/time.
      *
      *  @see mainDateTime()
@@ -982,6 +1064,8 @@ public:
      *                                     returns the deferral time.
      *                                     If a reminder has been deferred AND @p type
      *                                     contains NextReminder, returns the deferral time.
+     * - @p type contains NextSkip:        ignores all skipped occurrences, and their
+     *                                     reminders.
      *
      *  @param preDateTime  the date/time after which to find the next trigger/display.
      *  @param result       date/time of next trigger/display, or invalid date/time if none.
@@ -992,8 +1076,8 @@ public:
     TriggerType nextDateTime(const KADateTime& preDateTime, DateTime& result, NextTypes type, const KADateTime& endTime = {}) const;
 
     /** Return the next time the main alarm will trigger.
-     *  @note No account is taken of any working hours or holiday restrictions.
-     *        when determining the next trigger date/time.
+     *  @note No account is taken of any working hours or holiday restrictions,
+     *        or skipping, when determining the next trigger date/time.
      *
      *  @param withRepeats  true to include sub-repetitions, false to exclude them.
      *  @see mainTime(), startDateTime(), setTime()
@@ -1002,15 +1086,15 @@ public:
 
     /** Return the time at which the main alarm will next trigger.
      *  Sub-repetitions are ignored.
-     *  @note No account is taken of any working hours or holiday restrictions.
-     *        when determining the next trigger time.
+     *  @note No account is taken of any working hours or holiday restrictions,
+     *        or skipping, when determining the next trigger time.
      */
     QTime mainTime() const;
 
     /** Return the time at which the last sub-repetition of the current
      *  recurrence of the main alarm will occur.
-     *  @note No account is taken of any working hours or holiday restrictions
-     *        when determining the last sub-repetition time.
+     *  @note No account is taken of any working hours or holiday restrictions,
+     *        or skipping, when determining the last sub-repetition time.
      *
      *  @return last sub-repetition time, or main alarm time if no
      *          sub-repetitions are configured.
@@ -1031,10 +1115,12 @@ public:
     static void adjustStartOfDay(const KAEvent::List& events);
 
     /** Return the next time the alarm will trigger.
-     *  @param type specifies whether to ignore reminders, working time
-     *              restrictions, etc.
+     *  @param type  specifies whether to ignore reminders, working time
+     *               restrictions, etc.
+     *  @param skip  whether to take account of skipping.
+     *  @return next trigger time, or invalid if none.
      */
-    DateTime nextTrigger(Trigger type) const;
+    DateTime nextTrigger(Trigger type, bool skip = false) const;
 
     /** Set the date/time the event was created, or saved in the archive calendar.
      *  @see createdDateTime()
@@ -1060,6 +1146,21 @@ public:
      */
     bool repeatAtLogin(bool includeArchived = false) const;
 
+    /** Enable or disable the alarm to always execute when it is due, regardless
+     *  of any notification inhibition or active full screen application.
+     *  If disabled, the alarm's execution will be suspended until notifications
+     *  are enabled, and in the case of a display or audio alarm, execution will
+     *  be suspended while a full screen application is active.
+     *  @param exec  true to always execute, false to suspend execution
+     *  @see noInhibit()
+     */
+    void setNoInhibit(bool exec);
+
+    /** Return whether always-execute is enabled for the alarm.
+     *  @see setAlwaysExecute()
+     */
+    bool noInhibit() const;
+
     /** Enable or disable wake-from-suspend when the alarm is due.
      *  (Note that this option is used for kernel timer wake-from-suspend, not
      *   for RTC wake-from-suspend, since the latter can only be set for one
@@ -1288,8 +1389,8 @@ public:
     int recurInterval() const;
 
     /** Return the longest interval which can occur between consecutive recurrences.
-     *  @note No account is taken of any working hours or holiday restrictions
-     *        when evaluating consecutive recurrence dates/times.
+     *  @note No account is taken of any working hours or holiday restrictions,
+     *        or skipping, when evaluating consecutive recurrence dates/times.
      *  @see recurInterval()
      */
     KCalendarCore::Duration longestRecurrenceInterval() const;
@@ -1297,8 +1398,8 @@ public:
     /** Adjust the event date/time to the first recurrence of the event, on or after
      *  the event start date/time. The event start date may not be a recurrence date,
      *  in which case a later date will be set.
-     *  @note No account is taken of any working hours or holiday restrictions
-     *        when determining the first recurrence of the event.
+     *  @note No account is taken of any working hours or holiday restrictions,
+     *        or skipping, when determining the first recurrence of the event.
      */
     void setFirstRecurrence();
 
@@ -1319,11 +1420,11 @@ public:
     Repetition repetition() const;
 
     /** Return the count of the next sub-repetition which is due.
-     *  @note No account is taken of any working hours or holiday restrictions
-     *        when determining the next event sub-repetition.
+     *  @note No account is taken of any working hours or holiday restrictions,
+     *        or skipping, when determining the next event sub-repetition.
      *
      *  @return sub-repetition count (>=1), or 0 for the main recurrence.
-     *  @see nextOccurrence()
+     *  @see nextDateTime()
      */
     int nextRepetition() const;
 
@@ -1332,8 +1433,8 @@ public:
 
     /** Determine whether the event will occur strictly after the specified
      *  date/time. Reminders are ignored.
-     *  @note No account is taken of any working hours or holiday restrictions
-     *        when determining event occurrences.
+     *  @note No account is taken of any working hours or holiday restrictions,
+     *        or skipping, when determining event occurrences.
      *  @note If the event is date-only, its occurrences are considered to occur
      *        at the start-of-day time when comparing with @p preDateTime.
      *
@@ -1341,7 +1442,7 @@ public:
      *  @param includeRepetitions if true and the alarm has a sub-repetition, the
      *                            method will return true if any sub-repetitions
      *                            occur after @p preDateTime.
-     *  @see nextOccurrence()
+     *  @see nextDateTime()
      */
     bool occursAfter(const KADateTime& preDateTime, bool includeRepetitions) const;
 
@@ -1351,8 +1452,8 @@ public:
      *  If the alarm has a sub-repetition, and a sub-repetition of a previous
      *  recurrence occurs after the specified date/time, that sub-repetition is
      *  set as the next occurrence.
-     *  @note No account is taken of any working hours or holiday restrictions
-     *        when determining and setting the next occurrence date/time.
+     *  @note No account is taken of any working hours or holiday restrictions, or
+     *        skipping, when determining and setting the next occurrence date/time.
      *  @note If the event is date-only, its occurrences are considered to occur
      *        at the start-of-day time when comparing with @p preDateTime.
      *
@@ -1361,15 +1462,15 @@ public:
      *                      only None, Recur and Repeat values/bitmasks are valid;
      *                      other bit values should be disregarded.
      *  @return whether the event's next occurrence has been updated
-     *  @see nextOccurrence()
+     *  @see nextDateTime()
      */
     bool setNextOccurrence(const KADateTime& preDateTime, OccurType& type);
     void setNextOccurrence(const KADateTime& preDateTime);
 
     /** Get the date/time of the last previous occurrence of the event,
      *  strictly before the specified date/time. Reminders are ignored.
-     *  @note No account is taken of any working hours or holiday restrictions
-     *        when determining the previous event occurrence.
+     *  @note No account is taken of any working hours or holiday restrictions,
+     *        or skipping, when determining the previous event occurrence.
      *  @note If the event is date-only, its occurrences are considered to occur
      *        at the start-of-day time when comparing with @p preDateTime.
      *
@@ -1381,7 +1482,7 @@ public:
      *                                  last previous repetition is returned if
      *                                  appropriate;
      *                        Other values must not be specified.
-     *  @see nextOccurrence()
+     *  @see nextDateTime()
      */
     OccurType previousOccurrence(const KADateTime& afterDateTime, DateTime& result, Repeats includeRepetitions) const;
 
